package net.h34t.filemure

import java.time.LocalDateTime

data class AccountRef(
    val id: Long,
    val extId: ExtId,
    val email: String,
    val created: LocalDateTime,
    val state: State
)

@JvmInline
value class ExtId(val value: String) {

    companion object {
        private val chars = ('a'..'z') + ('A'..'Z') + ('0'..'9')

        fun generate(): ExtId {
            return ExtId((0..8).map { chars.random() }.joinToString(""))
        }
    }

    override fun toString() = value

}

data class Document(
    val id: Long,
    val extId: ExtId,
    val title: String,
    val description: String,
    val tags: List<Tag>,
    val created: LocalDateTime,
    val referenceDate: LocalDateTime,
    val state: State,
    val files: List<FileRef>,
)

@JvmInline
value class Tag(val value: String) {
    init {
        // TODO proper validation
        require(value.isNotBlank())
    }
}

data class FileRef(
    val id: Long,
    val extId: ExtId,
    val accountId: Long,
    val documentId: Long?,
    val filename: String,
    val contentType: String?,
    val contentExtracted: String?,
    val fileSize: Long,
    val created: LocalDateTime,
    val state: State
)

data class FileContent(
    val id: Long,
    val extId: ExtId,
    val filename: String,
    val contentType: String?,
    val contentExtracted: String?,
    val fileSize: Long,
    val content: ByteArray,
) {
    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (javaClass != other?.javaClass) return false

        other as FileContent

        if (id != other.id) return false
        if (extId != other.extId) return false
        if (!content.contentEquals(other.content)) return false

        return true
    }

    override fun hashCode(): Int {
        var result = id.hashCode()
        result = 31 * result + extId.hashCode()
        result = 31 * result + content.contentHashCode()
        return result
    }
}

enum class State(val code: Int) {
    ACTIVE(1),
    ARCHIVED(2),
    DELETED(3);

    companion object {
        fun fromCode(code: Int) = when (code) {
            1 -> ACTIVE
            2 -> ARCHIVED
            3 -> DELETED
            else -> throw IllegalArgumentException("$code doesn't exist.")
        }
    }
}

/**
 * Tuple for internal and external ID.
 */
data class IdPair(val id: Long, val extId: ExtId)